@@ -1,9 +1,9 @@
 """Generic SQL agent implementation for MCP tools."""
 
 from typing import AsyncIterator
+
 from sqlsaber.agents.base import BaseSQLAgent
 from sqlsaber.database.connection import BaseDatabaseConnection
-from sqlsaber.models.events import StreamEvent
 
 
 class MCPSQLAgent(BaseSQLAgent):
@@ -14,7 +14,7 @@ class MCPSQLAgent(BaseSQLAgent):
 
     async def query_stream(
         self, user_query: str, use_history: bool = True
-    ) -> AsyncIterator[StreamEvent]:
+    ) -> AsyncIterator:
         """Not implemented for generic agent as it's only used for tool operations."""
         raise NotImplementedError(
             "MCPSQLAgent does not support query streaming. Use specific agent implementations for conversation."
