@@ -2,21 +2,19 @@
 
 import asyncio
 import sys
-from pathlib import Path
 from typing import Annotated
 
 import cyclopts
 from rich.console import Console
 
-from sqlsaber.agents.anthropic import AnthropicSQLAgent
 from sqlsaber.cli.auth import create_auth_app
 from sqlsaber.cli.database import create_db_app
-from sqlsaber.cli.interactive import InteractiveSession
 from sqlsaber.cli.memory import create_memory_app
 from sqlsaber.cli.models import create_models_app
-from sqlsaber.cli.streaming import StreamingQueryHandler
+from sqlsaber.cli.threads import create_threads_app
+
+# Lazy imports - only import what's needed for CLI parsing
 from sqlsaber.config.database import DatabaseConfigManager
-from sqlsaber.database.connection import DatabaseConnection
 
 
 class CLIError(Exception):
@@ -29,9 +27,14 @@ class CLIError(Exception):
 
 app = cyclopts.App(
     name="sqlsaber",
-    help="SQLSaber - Use the agent Luke!\n\nSQL assistant for your database",
+    help="SQLsaber - Open-source agentic SQL assistant for your database",
 )
 
+app.command(create_auth_app(), name="auth")
+app.command(create_db_app(), name="db")
+app.command(create_memory_app(), name="memory")
+app.command(create_models_app(), name="models")
+app.command(create_threads_app(), name="threads")
 
 console = Console()
 config_manager = DatabaseConfigManager()
@@ -43,7 +46,7 @@ def meta_handler(
         str | None,
         cyclopts.Parameter(
             ["--database", "-d"],
-            help="Database connection name or direct file path for CSV/SQLite files (uses default if not specified)",
+            help="Database connection name, file path (CSV/SQLite), or connection string (postgresql://, mysql://) (uses default if not specified)",
         ),
     ] = None,
 ):
@@ -56,6 +59,8 @@ def meta_handler(
         saber -d mydb "show me users"          # Run a query with specific database
         saber -d data.csv "show me users"      # Run a query with ad-hoc CSV file
         saber -d data.db "show me users"       # Run a query with ad-hoc SQLite file
+        saber -d "postgresql://user:pass@host:5432/db" "show users"  # PostgreSQL connection string
+        saber -d "mysql://user:pass@host:3306/db" "show users"       # MySQL connection string
         echo "show me all users" | saber       # Read query from stdin
         cat query.txt | saber                  # Read query from file via stdin
     """
@@ -75,7 +80,7 @@ def query(
         str | None,
         cyclopts.Parameter(
             ["--database", "-d"],
-            help="Database connection name or direct file path for CSV/SQLite files (uses default if not specified)",
+            help="Database connection name, file path (CSV/SQLite), or connection string (postgresql://, mysql://) (uses default if not specified)",
         ),
     ] = None,
 ):
@@ -92,10 +97,27 @@ def query(
         saber "show me all users"         # Run a single query
         saber -d data.csv "show users"    # Run a query with ad-hoc CSV file
         saber -d data.db "show users"     # Run a query with ad-hoc SQLite file
+        saber -d "postgresql://user:pass@host:5432/db" "show users"  # PostgreSQL connection string
+        saber -d "mysql://user:pass@host:3306/db" "show users"       # MySQL connection string
         echo "show me all users" | saber  # Read query from stdin
     """
 
     async def run_session():
+        # Import heavy dependencies only when actually running a query
+        # This is only done to speed up startup time
+        from sqlsaber.agents import build_sqlsaber_agent
+        from sqlsaber.cli.interactive import InteractiveSession
+        from sqlsaber.cli.streaming import StreamingQueryHandler
+        from sqlsaber.database.connection import (
+            CSVConnection,
+            DatabaseConnection,
+            MySQLConnection,
+            PostgreSQLConnection,
+            SQLiteConnection,
+        )
+        from sqlsaber.database.resolver import DatabaseResolutionError, resolve_database
+        from sqlsaber.threads import ThreadStorage
+
         # Check if query_text is None and stdin has data
         actual_query = query_text
         if query_text is None and not sys.stdin.isatty():
@@ -105,39 +127,13 @@ def query(
                 # If stdin was empty, fall back to interactive mode
                 actual_query = None
 
-        # Get database configuration or handle direct file paths
-        if database:
-            # Check if this is a direct CSV file path
-            if database.endswith(".csv"):
-                csv_path = Path(database).expanduser().resolve()
-                if not csv_path.exists():
-                    raise CLIError(f"CSV file '{database}' not found.")
-                connection_string = f"csv:///{csv_path}"
-                db_name = csv_path.stem
-            # Check if this is a direct SQLite file path
-            elif database.endswith((".db", ".sqlite", ".sqlite3")):
-                sqlite_path = Path(database).expanduser().resolve()
-                if not sqlite_path.exists():
-                    raise CLIError(f"SQLite file '{database}' not found.")
-                connection_string = f"sqlite:///{sqlite_path}"
-                db_name = sqlite_path.stem
-            else:
-                # Look up configured database connection
-                db_config = config_manager.get_database(database)
-                if not db_config:
-                    raise CLIError(
-                        f"Database connection '{database}' not found. Use 'sqlsaber db list' to see available connections."
-                    )
-                connection_string = db_config.to_connection_string()
-                db_name = db_config.name
-        else:
-            db_config = config_manager.get_default_database()
-            if not db_config:
-                raise CLIError(
-                    "No database connections configured. Use 'sqlsaber db add <name>' to add a database connection."
-                )
-            connection_string = db_config.to_connection_string()
-            db_name = db_config.name
+        # Resolve database from CLI input
+        try:
+            resolved = resolve_database(database, config_manager)
+            connection_string = resolved.connection_string
+            db_name = resolved.name
+        except DatabaseResolutionError as e:
+            raise CLIError(str(e))
 
         # Create database connection
         try:
@@ -145,25 +141,60 @@ def query(
         except Exception as e:
             raise CLIError(f"Error creating database connection: {e}")
 
-        # Create agent instance with database name for memory context
-        agent = AnthropicSQLAgent(db_conn, db_name)
+        # Create pydantic-ai agent instance with database name for memory context
+        agent = build_sqlsaber_agent(db_conn, db_name)
 
         try:
             if actual_query:
                 # Single query mode with streaming
                 streaming_handler = StreamingQueryHandler(console)
-                console.print(
-                    f"[bold blue]Connected to:[/bold blue] {db_name} {agent._get_database_type_name()}\n"
+                # Compute DB type for the greeting line
+                db_type = (
+                    "PostgreSQL"
+                    if isinstance(db_conn, PostgreSQLConnection)
+                    else "MySQL"
+                    if isinstance(db_conn, MySQLConnection)
+                    else "SQLite"
+                    if isinstance(db_conn, (SQLiteConnection, CSVConnection))
+                    else "database"
                 )
-                await streaming_handler.execute_streaming_query(actual_query, agent)
+                console.print(
+                    f"[bold blue]Connected to:[/bold blue] {db_name} ({db_type})\n"
+                )
+                run = await streaming_handler.execute_streaming_query(
+                    actual_query, agent
+                )
+                # Persist non-interactive run as a thread snapshot so it can be resumed later
+                try:
+                    if run is not None:
+                        threads = ThreadStorage()
+                        # Extract title and model name
+                        title = actual_query
+                        model_name: str | None = agent.model.model_name
+
+                        thread_id = await threads.save_snapshot(
+                            messages_json=run.all_messages_json(),
+                            database_name=db_name,
+                        )
+                        await threads.save_metadata(
+                            thread_id=thread_id, title=title, model_name=model_name
+                        )
+                        await threads.end_thread(thread_id)
+                        console.print(
+                            f"[dim]You can continue this thread using:[/dim] saber threads resume {thread_id}"
+                        )
+                except Exception:
+                    # best-effort persistence; don't fail the CLI on storage errors
+                    pass
+                finally:
+                    await threads.prune_threads()
             else:
                 # Interactive mode
-                session = InteractiveSession(console, agent)
+                session = InteractiveSession(console, agent, db_conn, db_name)
                 await session.run()
 
         finally:
             # Clean up
-            await agent.close()  # Close the agent's HTTP client
             await db_conn.close()
             console.print("\n[green]Goodbye![/green]")
 
@@ -175,23 +206,6 @@ def query(
         sys.exit(e.exit_code)
 
 
-# Add authentication management commands
-auth_app = create_auth_app()
-app.command(auth_app, name="auth")
-
-# Add database management commands after main callback is defined
-db_app = create_db_app()
-app.command(db_app, name="db")
-
-# Add memory management commands
-memory_app = create_memory_app()
-app.command(memory_app, name="memory")
-
-# Add model management commands
-models_app = create_models_app()
-app.command(models_app, name="models")
-
-
 def main():
     """Entry point for the CLI application."""
     app()
