@@ -93,12 +93,15 @@ class LiveMarkdownRenderer:
         """Finalize and stop the current Live segment, if any."""
         if self._live is None:
             return
-        if self._buffer:
-            self._live.update(Markdown(self._buffer))
+        # Persist the *final* render exactly once, then shut Live down.
+        buf = self._buffer
         self._live.stop()
         self._live = None
         self._buffer = ""
         self._current_kind = None
+        # Print the complete markdown to scroll-back for permanent reference
+        if buf:
+            self.console.print(Markdown(buf))
 
     def end_if_active(self) -> None:
         self.end()
@@ -154,10 +157,12 @@ class LiveMarkdownRenderer:
         if self._live is not None:
             self.end()
         self._buffer = initial_markdown or ""
+        # NOTE: Use transient=True so the live widget disappears on exit,
+        # giving a clean transition to the final printed result.
         live = Live(
             Markdown(self._buffer),
             console=self.console,
-            vertical_overflow="visible",
+            transient=True,
             refresh_per_second=12,
         )
         self._live = live
@@ -193,22 +198,32 @@ class DisplayManager:
         # Normalized leading blank line before tool headers
         self.show_newline()
         if tool_name == "list_tables":
-            self.console.print(
-                "[dim bold]:gear: Discovering available tables[/dim bold]"
-            )
+            if self.console.is_terminal:
+                self.console.print(
+                    "[dim bold]:gear: Discovering available tables[/dim bold]"
+                )
+            else:
+                self.console.print("**Discovering available tables**\n")
         elif tool_name == "introspect_schema":
             pattern = tool_input.get("table_pattern", "all tables")
-            self.console.print(
-                f"[dim bold]:gear: Examining schema for: {pattern}[/dim bold]"
-            )
+            if self.console.is_terminal:
+                self.console.print(
+                    f"[dim bold]:gear: Examining schema for: {pattern}[/dim bold]"
+                )
+            else:
+                self.console.print(f"**Examining schema for:** {pattern}\n")
         elif tool_name == "execute_sql":
             # For streaming, we render SQL via LiveMarkdownRenderer; keep Syntax
             # rendering for threads show/resume. Controlled by include_sql flag.
             query = tool_input.get("query", "")
-            self.console.print("[dim bold]:gear: Executing SQL:[/dim bold]")
-            self.show_newline()
-            syntax = Syntax(query, "sql", background_color="default", word_wrap=True)
-            self.console.print(syntax)
+            if self.console.is_terminal:
+                self.console.print("[dim bold]:gear: Executing SQL:[/dim bold]")
+                self.show_newline()
+                syntax = Syntax(query, "sql", background_color="default", word_wrap=True)
+                self.console.print(syntax)
+            else:
+                self.console.print("**Executing SQL:**\n")
+                self.console.print(f"```sql\n{query}\n```\n")
 
     def show_text_stream(self, text: str):
         """Display streaming text."""
@@ -220,9 +235,12 @@ class DisplayManager:
         if not results:
             return
 
-        self.console.print(
-            f"\n[bold magenta]Results ({len(results)} rows):[/bold magenta]"
-        )
+        if self.console.is_terminal:
+            self.console.print(
+                f"\n[bold magenta]Results ({len(results)} rows):[/bold magenta]"
+            )
+        else:
+            self.console.print(f"\n**Results ({len(results)} rows):**\n")
 
         # Create table with columns from first result
         all_columns = list(results[0].keys())
@@ -230,9 +248,14 @@ class DisplayManager:
 
         # Show warning if columns were truncated
         if len(all_columns) > 15:
-            self.console.print(
-                f"[yellow]Note: Showing first 15 of {len(all_columns)} columns[/yellow]"
-            )
+            if self.console.is_terminal:
+                self.console.print(
+                    f"[yellow]Note: Showing first 15 of {len(all_columns)} columns[/yellow]"
+                )
+            else:
+                self.console.print(
+                    f"*Note: Showing first 15 of {len(all_columns)} columns*\n"
+                )
 
         table = self._create_table(display_columns)
 
@@ -243,14 +266,28 @@ class DisplayManager:
         self.console.print(table)
 
         if len(results) > 20:
-            self.console.print(
-                f"[yellow]... and {len(results) - 20} more rows[/yellow]"
-            )
+            if self.console.is_terminal:
+                self.console.print(
+                    f"[yellow]... and {len(results) - 20} more rows[/yellow]"
+                )
+            else:
+                self.console.print(
+                    f"*... and {len(results) - 20} more rows*\n"
+                )
 
     def show_error(self, error_message: str):
         """Display error message."""
         self.console.print(f"\n[bold red]Error:[/bold red] {error_message}")
 
+    def show_sql_error(self, error_message: str, suggestions: list[str] | None = None):
+        """Display SQL-specific error with optional suggestions."""
+        self.show_newline()
+        self.console.print(f"[bold red]SQL error:[/bold red] {error_message}")
+        if suggestions:
+            self.console.print("[yellow]Hints:[/yellow]")
+            for suggestion in suggestions:
+                self.console.print(f"  • {suggestion}")
+
     def show_processing(self, message: str):
         """Display processing message."""
         self.console.print()  # Add newline
@@ -371,6 +408,13 @@ class DisplayManager:
                     for fk in foreign_keys:
                         self.console.print(f"  • {fk}")
 
+                # Show indexes
+                indexes = table_info.get("indexes", [])
+                if indexes:
+                    self.console.print("[bold blue]Indexes:[/bold blue]")
+                    for idx in indexes:
+                        self.console.print(f"  • {idx}")
+
         except json.JSONDecodeError:
             self.show_error("Failed to parse schema data")
         except Exception as e:
